package cn.com.ctop.performanceappraisal.service.impl;

import cn.com.ctop.performanceappraisal.entity.OptimizerConfig;
import cn.com.ctop.performanceappraisal.entity.Performance;
import cn.com.ctop.performanceappraisal.entity.PerformanceOptimizer;
import cn.com.ctop.performanceappraisal.mapper.PerformanceMapper;
import cn.com.ctop.performanceappraisal.mapper.PerformanceOptimizerMapper;
import cn.com.ctop.performanceappraisal.service.IOptimizerConfigService;
import cn.com.ctop.performanceappraisal.service.IPerformanceService;
import cn.com.ctop.performanceappraisal.vo.DesignerCostDetailVO;
import cn.com.ctop.performanceappraisal.vo.OptimizerCostDetailVO;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 设计师季度绩效表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-12-10
 */
@Service
public class PerformanceServiceImpl extends ServiceImpl<PerformanceMapper, Performance> implements IPerformanceService {
    @Autowired
    private PerformanceMapper performanceMapper;
    @Autowired
    private IOptimizerConfigService optimizerConfigService;

    @Override
    public List<DesignerCostDetailVO> getTotalCostDetailByRoleId(String roleId) {
        return null;
    }

    @Override
    public List<OptimizerCostDetailVO> getOptimizerTotalCostDetail(String startDate, String endDate, String year, String quarter) {
        return performanceMapper.getOptimizerTotalCostDetail(startDate, endDate, year, quarter);
    }

    /**
     * 计算设计岗位绩效逻辑
     * 1：查询用户总消耗以及总视频数
     * 2：查询视频有效率排行榜
     * 3：根据视频有效率分配提点比例
     * 4：计算个人总绩效
     */

    /**
     * 计算优化师岗位绩效逻辑
     * 1：查询所有优化师总消耗数据
     * 2：查询媒体季度任务完成情况
     * 3：计算单个优化师提成总额
     */
    public List<OptimizerCostDetailVO> getOptimizerPerformance() {
        Date date = new Date();
        Date getDate = DateUtils.addDay(date, -1);
        String startDate = DateUtils.getQuarterStartDate(getDate);
        String endDate = DateUtils.getQuarterEndDate(getDate);
        String year = DateUtils.getYear(getDate) + "";
        String quarter = DateUtils.getQuarter(getDate) + "";
        //1：查询所有优化师总消耗数据
        List<OptimizerCostDetailVO> detailVOS = getOptimizerTotalCostDetail(startDate, endDate, year, quarter);
        //查询优化师系统设置数据
        OptimizerConfig config = optimizerConfigService.getEnabledConfigByMediaType("2");
        //2: 查询媒体季度任务完成情况
        OptimizerCostDetailVO totalCostVo = performanceMapper.getOptimizerTotalCost(startDate, endDate, year, quarter);
        //3：计算单个优化师提成总额
        List<OptimizerCostDetailVO> getDetailVos = new ArrayList<>();
        if (null != detailVOS && detailVOS.size() > 0) {
            for (OptimizerCostDetailVO vo : detailVOS) {
                vo.setTotalRoyalty(getTotalRoyalty(config, vo, totalCostVo));
                PerformanceOptimizer optimizer = new PerformanceOptimizer(vo);
                optimizerMapper.insert(optimizer);
            }
        }
        //TODO 需要入库
        return getDetailVos;
    }

    @Autowired
    private PerformanceOptimizerMapper optimizerMapper;

    private BigDecimal getTotalRoyalty(OptimizerConfig config, OptimizerCostDetailVO vo, OptimizerCostDetailVO quarterCost) {
        BigDecimal royalty = BigDecimal.ZERO;
        //个人总消耗
        BigDecimal totalCost = vo.getTotalCost();
        //个人季度任务
        BigDecimal personalTask = config.getPersonalTask();
        //媒体季度任务
        BigDecimal mediaTask = config.getMediaTask();
        //媒体任务未完成提点
        BigDecimal mediaTaskNoFinishRate = config.getMediaTaskNoFinishRate();
        //媒体任务未完成提点
        BigDecimal mediaTaskFinishRate = config.getMediaTaskFinishRate();
        //个人任务未完成
        if (totalCost.compareTo(personalTask) < 0) {
            return royalty;
        }
        //季度任务未完成
        if (totalCost.compareTo(mediaTask) < 0) {
            return totalCost.multiply(mediaTaskNoFinishRate).divide(new BigDecimal("1000")).setScale(2, RoundingMode.HALF_UP);
        }

        return totalCost.multiply(mediaTaskFinishRate).divide(new BigDecimal("1000")).setScale(2, RoundingMode.HALF_UP);
    }

}
